from win32com import client as vba

# Constantes do Word

wdNewBlankDocument = 0
wdExtend = 1
wdCharacter = 1
wdWord = 2
wdLine = 5
wdStory = 6

# Constantes do Excel

xlExcel12 = 50
xlOpenXMLWorkbook = 51
xlOpenXMLWorkbookMacroEnabled = 52
xlExcel8 = 56
xlCSVUTF8 = 62

# Funções do Word


def getNewWordApplication(visible=False):
    import platform
    if platform.system() == 'Windows':
        word = vba.DispatchEx('Word.Application')
        word.Visible = visible
        return word
    else:
        print(ValueError('Not implemented for other platforms than Windows.'))


def openWordDocument(word_filename=None, visible=False):
    import platform
    if platform.system() == 'Windows':
        word = vba.DispatchEx('Word.Application')
        word.Visible = visible
        word_document = word.Documents.Open(word_filename)
        return word, word_document
    else:
        print(ValueError('Not implemented for other platforms than Windows.'))


def convertWordToPDF(word_filename=None, pdf_filename=None):
    import platform
    if platform.system() == 'Windows':
        word = vba.DispatchEx('Word.Application')
        word_document = word.Documents.Open(word_filename)
        word_document.SaveAs(pdf_filename, FileFormat=17)
        word_document.Close()
    else:
        print(ValueError('Not implemented for other platforms than Windows.'))


# Funções do Excel


def getNewExcelApplication(visible=False):
    import platform
    if platform.system() == 'Windows':
        xl = vba.DispatchEx('Excel.Application')
        xl.Visible = visible
        return xl
    else:
        print(ValueError('Not implemented for other platforms than Windows.'))


def openExcelWorkbook(excel_filename=None, visible=False):
    import platform
    if platform.system() == 'Windows':
        xl = vba.DispatchEx('Excel.Application')
        xl.Visible = visible
        xl_plan = xl.Workbooks.Open(excel_filename)
        return xl, xl_plan
    else:
        print(ValueError('Not implemented for other platforms than Windows.'))



from time import sleep
from getpass import getuser as username

from platform import machine as architecture
from platform import node as hostname
from platform import system

from os import getcwd
from os import startfile
from os import chdir as folderSelect
from os import mkdir as folderCreate
from os import rmdir as folderRemove
from os import rename as folderRename
from os.path import isfile as fileExist
from os.path import isdir as folderExist
from shutil import make_archive as folderZip


def ip_externo(url=''):
    from autotasks import web
    soup = web.getSoupHtmlParser(url)
    rst_soup = soup.findAll('h3', {'class', 'm-0 font-weight-bold'})
    return web.re_search(r'[0-9]+.[0-9]+.[0-9]+.[0-9]+', rst_soup.__str__())[0]


def processRunning(name):
    # Checks if given process name (name) is currently running on the system. Returns True or False.
    if name:
        import psutil
        for p in psutil.process_iter():
            if name in p.name():
                return True
    return False


def listRunningProcesses():
    # Returns a list with all names of unique processes currently running on the system.
    process_list = []
    import psutil
    for p in psutil.process_iter():
        process_list.append(p.name())
    return set(process_list)


def folderUnzip(path, new_path=False):
    import os
    import zipfile
    if os.path.exists(path):
        zipp = zipfile.ZipFile(path)
        if not new_path:
            base_path = "\\".join(path.split("\\")[:-1])
            zipp.extractall(base_path)
        elif os.path.isdir(path):
            zipp.extractall(path)
        zipp.close()
    return


def waitForFile(path):
    # Wait until a file with the entered path exists.
    import os
    from time import sleep
    while not os.path.exists(path):
        sleep(1)
    return


def getUniqueKey128bits(length=36):
    # Universally unique identifier (UUID) is a 128-bit number used to identify information in computer systems.
    # This key can be considered as unique for there to be a one in a billion chance of duplication, 103 trillion
    # version 4 UUIDs must be generated. The general form is e.g. "123e4567-e89b-12d3-a456-4266554". The argument
    # specifies the length of the returned string. If it is omitted, the entire 128-bit UUID is returned.
    import uuid
    return str(uuid.uuid4())[:length]



from re import search as re_search      # Regex


def getSoupHtmlParser(url):
    # Retorna o Soup html parser do BeautifulSoup atraves do request de um site
    import requests
    from bs4 import BeautifulSoup
    r = requests.get(url)
    return BeautifulSoup(r.content, "html.parser")


def getChromeBrowser(url=False, ignore_images=False):  # , headless=False):
    # ---------------------------------------------------------------
    #     Abre o browser Chrome em um instancia Selenium.
    # ---------------------------------------------------------------
    #     Argumentos:
    #         ignore_images (bool): nao abre imagens
    #         headless (bool): executa sem janela
    #     Retorno:
    #         webdriver: Selenium Webdriver
    #     Exemplo:
    #         browser = ChromeBrowser(ignore_images=True)
    #         browser.get('https://github.com')
    # ---------------------------------------------------------------

    import os
    import platform

    if platform.system() == 'Linux':
        chromedriver_path = '\\bin\\webdriver\\linux64\\chromedriver'
    elif platform.system() == 'Windows':
        chromedriver_path = '\\bin\\win32\\chromedriver.exe'
    else:
        chromedriver_path = '\\bin\\mac64\\chromedriver.exe'

    from selenium.webdriver import Chrome, ChromeOptions

    chrome_options = ChromeOptions()

    # if headless:
    #    chrome_options.add_argument("--headless")

    if ignore_images:
        prefs = {"profile.managed_default_content_settings.images": 2}
        chrome_options.add_experimental_option("prefs", prefs)

    s_path = os.path.abspath(__file__)
    s_file = s_path[:s_path.rfind("\\") + 1]
    browser = Chrome(s_file + chromedriver_path, chrome_options=chrome_options)

    if url:
        browser.get(url)

    return browser


def chromeRunning():
    # Retorna True quando o Chrome esta executanto.
    import psutil
    for p in psutil.process_iter():
        if "chrome.exe" in p.name():
            return True
    return False



from pyautogui import password, confirm, alert, prompt


def credencial(mask='*', timeout=None):
    import getpass
    username = getpass.getuser()
    return password(text=username, title="Informe sua senha:", mask=mask, timeout=timeout)


def input(text='', title='', default='', root=None, timeout=None):
    result = prompt(text=text, title=title, default=default, root=root, timeout=timeout)
    return result



# coding=utf-8


from penknife import msg       # MsgBox tasks
from penknife import pdf       # PDF tasks
from penknife import rpa       # Robotics tasks
from penknife import web       # Web tasks
from penknife import sys       # System tasks
from penknife import vba       # Excel e Word tasks



from time import sleep                                                                                  # Delay
from pyautogui import press, hotkey, typewrite                                                          # Keyboard

# moveRel: Moves the mouse an x- and y- distance relative to its current pixel position.
# dragTo: Drag the mouse from its current position to a entered x-y position, while holding a specified button.
from pyautogui import position, mouseUp, mouseDown, moveTo, moveRel, vscroll, hscroll, dragTo, dragRel  # Mouse


def click(x=None, y=None, interval=0.0, duration=0.0, pause=0.0,
          button=['left', 'right', 'double'][0], screenshot=None):

    from pyautogui import leftClick, rightClick, doubleClick

    if button == 'right':
        rightClick(x=x, y=y, interval=interval, duration=duration, pause=pause, logScreenshot=screenshot)
    elif button == 'double':
        doubleClick(x=x, y=y, interval=interval, duration=duration, pause=pause, logScreenshot=screenshot)
    else:
        leftClick(x=x, y=y, interval=interval, duration=duration, pause=pause, logScreenshot=screenshot)


def clickOnImage(filename=None, button=['left', 'right', 'double'][0]):
    # Clica com o botao esquerdo, direito ou duplo click sobre uma imagem...
    from pyautogui import locateCenterOnScreen

    try:
        x, y = locateCenterOnScreen(filename)

        if button not in ('right', 'double'):
            click(x=x, y=y, button='left')
        else:
            click(x=x, y=y, button=button)

        return x, y

    except Exception:
        return None


def waitOnImage(filename=None, timeout=30, button=['left', 'right', 'double'][0]):
    # Aguarda ate que uma determinada imagem apareca na tela...
    from pyautogui import locateCenterOnScreen
    for _ in range(timeout):
        try:
            x, y = locateCenterOnScreen(filename)

            if button not in ('right', 'double'):
                click(x=x, y=y, button='left')
            else:
                click(x=x, y=y, button=button)

            return x, y

        except TypeError:
            sleep(1)


def carregaImage(img_file):
    from PIL import Image
    return Image.open(img_file)


def extractTextFromImage(img_file):
    from autotasks import sys
    if sys.system() == 'Windows':
        from pytesseract import pytesseract
        if sys.architecture() == 'x86':
            pytesseract.tesseract_cmd = r'C:\Program Files (x86)\Tesseract-OCR\tesseract'
        else:
            pytesseract.tesseract_cmd = r'C:\Program Files\Tesseract-OCR\tesseract'

    from PIL import Image
    from pytesseract import image_to_string
    return image_to_string(Image.open(img_file))


def typeInRunWindow(text=None):
    # Abre o "Run" do windows e escreve algo.
    if text:
        hotkey("win", "r")
        typewrite(text, 0.08)
        press("enter")
    return



import PyPDF2


def mergePDF(pdf1, pdf2, merged_path):

    # Os dois primeiros argumentos sao os PDFs que precisam ser mesclados, inseridos como um caminho. As paginas do
    # pdf2 serao adicionadas ao pdf2. O PDF mesclado recebe um novo caminho especificado pelo terceiro argumento.

    from PyPDF2 import PdfFileMerger

    pdfs = [str(pdf1), str(pdf2)]
    merger = PdfFileMerger()

    for pdf in pdfs:
        merger.append(pdf)

    merger.write(merged_path)
    return


def extractTextFromPDFPage(pdf_file, txt_file=None, bgn_page=1, end_page=1):

    # Extrai todo o texto de uma determinada pagina e o retorna como uma string. O pdf precisa ser inserido
    # como um caminho. Preste atencao que a pagina inserida precisa ser maior que 0.

    import os
    if os.path.isfile(pdf_file):

        pdf_reader = PyPDF2.PdfFileReader(open(pdf_file, "rb"))

        bgn_page = max(bgn_page, 1)
        end_page = max(end_page, 1)

        if end_page > pdf_reader.numPages:
            end_page = pdf_reader.numPages

        if bgn_page > pdf_reader.numPages:
            bgn_page = pdf_reader.numPages
        elif bgn_page > end_page:
            bgn_page = end_page

        page_content = ''
        for i_page in range(bgn_page - 1, end_page):
            page = pdf_reader.getPage(i_page)
            page_content += page.extractText()

        with open(txt_file, 'w') as f:
            f.write(page_content.__str__())
        f.close()

        return page_content



# coding=UTF-8

from setuptools import setup

setup(
    name='penknife',
    version='2021.05.04',
    author='Marcelo Horita',
    author_email='datazeus.tecnologia@gmail.com.br',
    packages=['penknife'],
    description='Pacote de automação de processos',
    long_description='Pacote que ajuda automatizar processos web, sys, rpa, vba, msg e pdf',
    url='https://github.com/mfhorita',
    license='GNU',
    keywords='penknife autotasks vba rpa msg pdf web scrapy sys',
    classifiers=[
        'Natural Language :: Portuguese (Brazilian)',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Internationalization',
        'Topic :: Scientific/Engineering :: Physics'
    ],
    install_requires=[
        'PyAutoGUI>=0.9.47',
        'beautifulsoup4>=4.8.0',
        'selenium>=3.7.0',
        'pytesseract>=0.3.0',
        'psutil>=5.7.0',
        'pywin32>=223',
        'PyPDF2>=1.26.0'
    ]
)
